from pyg_base import ulist, dictable,Dict, pd_read_parquet, eq
from pyg_mongo import mongo_reader, mongo_cursor, q, mongo_table, mongo_pk_cursor
from pyg_base import *
import numpy as np; import pandas as pd
import jsonpickle as jp
import pytest

    
def test_mongo_cursor():
    t = mongo_table('test', 'test')
    t.drop()
    t.insert_one(dict(a=1,b=2))
    assert len(t) == 1
    t.insert_one(dict(a=2,b=3))
    assert len(t) == 2    
    res = t[::]    

    assert isinstance(res, dictable) and len(res) == 2
    assert res['a','b'][0] == (1, 2)
    assert res['a','b'][1] == (2, 3)
    
    assert 'b' not in t['a'][::]
    assert len(t.inc(a =1)) == 1
    t.drop()

def test_mongo_pk_cursor():
    t = mongo_table('test', 'test')
    t.drop()
    self  = mongo_table('test', 'test', pk = 'key')    
    assert len(self) == 0
    self.update_one(dict(key = 1, data = 1))
    assert len(self) == 1
    assert len(t) == 1    
    
    doc = dict(key = 1, data = 2)
    self.update_one(doc)
    assert len(self) == 1
    assert len(t.deleted) == 1

    doc = dict(key = 1, other_data = 3)
    self.update_one(doc)
    assert len(self) == 1
    assert len(t.deleted) == 2
    
    assert self[dict(key=1)]['other_data'] == 3
    assert self[dict(key=1)]['data'] == 2
    self.reset.drop()
    assert len(t) == 0


def test_mongo_pk_cursor_multiple_keys():
    t = mongo_table('test', 'test')
    t.drop()
    self  = mongo_table('test', 'test', pk = ['a','b'])    
    with pytest.raises(ValueError):
        self.insert_one(dict(a=1,c=2))    
    self.insert_one(dict(a=1,b=2))
    assert type(self[0]) == dict
    doc = Dict(a=1,b=2)
    self._write(doc)
    self.update_one(doc)
    assert type(self[0]) == Dict
    self.reset.drop()

    
    
def test_mongo_table_mode():
    with pytest.raises(KeyError):
        mongo_table('test', 'test', mode = 'not good')
        
    r = mongo_table('test', 'test', mode = mongo_reader)
    assert isinstance(r, mongo_reader)


    w = mongo_table('test', 'test', mode = mongo_cursor)
    assert isinstance(w, mongo_cursor)
    
    with pytest.raises(ValueError):
        mongo_table('test', 'test', mode = mongo_pk_cursor)


def test_mongo_cursor_root():
    t = mongo_table('test', 'test', writer = 'c:/temp/%name/%surname.parquet')
    t.drop()
    doc = dict(name = 'adam', surname = 'smith', ts = pd.Series(np.arange(10)))
    t.insert_one(doc)
    assert eq(pd_read_parquet('c:/temp/adam/smith/ts.parquet'), doc['ts'])
    assert eq(t[0]['ts'], doc['ts'])
    doc = dict(name = 'beth', surname = 'brown', a = np.arange(10))
    t.drop()
    t.insert_one(doc)
    assert eq(np.load('c:/temp/beth/brown/a.npy'), doc['a'])
    assert eq(t[0]['a'], doc['a'])
    t.drop()

    
    


from pyg_base import *
from pyg_mongo import * 

# import Dict, pd_read_parquet, parquet_write, mongo_table, dictable, eq, passthru, cell, drange, root_path, dt, parquet_encode, csv_encode
import pandas as pd
import pytest
from functools import partial

def test_parquet_writer():
    doc = Dict(key = 'a', data = dictable(a = [pd.Series([1,2,3]), pd.Series([4,5,6])], b = [1,2]), other = Dict(df = pd.DataFrame(dict(a=[1,2,3], b= [4,5,6]))))
    t = mongo_table('test', 'test')
    t.drop()
    p = mongo_table('test', 'test', writer = 'c:/temp/%key.parquet')
    p.writer
    _ = p.insert_one(doc)
    assert eq(p[0]['data'], doc['data'])
    assert eq(t[0]['other'], doc['other'])
    d = t.read(0, passthru)    
    assert d['data']['a'][0]['path'] == 'c:/temp/a/data/a/0.parquet'
    assert d['other']['df']['path'] == 'c:/temp/a/other/df.parquet'
    p.drop()
    assert len(p) == 0
    
def test_csv_writer():
    doc = Dict(key = 'a', data = dictable(a = [pd.Series([1,2,3]), pd.Series([4,5,6])], b = [1,2]), other = Dict(df = pd.DataFrame(dict(a=[1,2,3], b= [4,5,6]))))
    t = mongo_table('test', 'test')
    t.drop()
    p = mongo_table('test', 'test', writer = 'c:/temp/%key.csv')
    p.writer
    _ = p.insert_one(doc)
    assert eq(p[0]['data'], doc['data'])
    assert eq(t[0]['other'], doc['other'])
    d = t.read(0, passthru)    
    assert d['data']['a'][0]['path'] == 'c:/temp/a/data/a/0.csv'
    assert d['other']['df']['path'] == 'c:/temp/a/other/df.csv'
    p.drop()
    assert len(p) == 0


def test_root_path():
    root = 'c:/%school/%pupil.name/%pupil.surname/' 
    doc = dict(school = 'kings',  
               pupil = dict(name = 'yoav', surname = 'git'),  
               grades = dict(maths = 100, physics = 20, chemistry = 80),  
               report = dict(date = dt(2000,1,1),  
                             teacher = dict(name = 'adam', surname = 'cohen') 
                             ) 

                ) 

    assert root_path(doc, root) == 'c:/kings/yoav/git/' 
    root = 'c:/%school/%pupil.name_%pupil.surname/' 
    assert root_path(doc, root) == 'c:/kings/yoav_git/' 
    root = 'c:/archive/%report.date/%pupil.name.%pupil.surname/' 
    assert root_path(doc, root, '%Y') == 'c:/archive/2000/yoav.git/'  # can choose to format dates by providing a fmt. 


    root = 'c:/%name/%surname/%age/'
    doc = dict(name = 'yoav', surname = 'git')
    with pytest.raises(ValueError):
        root_path(doc, root)

def test_parquet_encode():
    value = dict(a = 1)
    path = 'c:/.parquet'
    assert parquet_encode(value, path) == value
    assert csv_encode(value, path) == value

def test_parquet_write():
    s = pd.Series([1,2,3], drange(2))
    db_with_root = partial(mongo_table, db = 'test', table = 'test', writer = 'c:/temp/test.parquet')
    db_no_root = partial(mongo_table, db = 'test', table = 'test', writer = '.parquet')
    c = parquet_write(dict(a = 1, data = s, root = 'c:/temp/test/'))
    assert eq(pd_read_parquet('c:/temp/test/data.parquet'), s)
    c = parquet_write(dict(a = 1, other_data = s, db = db_with_root), root = 'c:/temp/test')
    assert eq(pd_read_parquet('c:/temp/test/other_data.parquet'), s)
    c = parquet_write(dict(a = 1, data = s, db = db_no_root))
    assert eq(c['data'], s)




from pyg_base import dictable, Dict, drange, pd_read_parquet, passthru, eq, dt
from pyg_mongo import periodic_cell, mongo_table, mongo_pk_cursor, mongo_cursor, mongo_reader
from functools import partial
import pytest
import pandas as pd
_pk = 'pk'

def test_mongo_pk_cursor():
    db = partial(mongo_table, db = 'test', table = 'test', pk = ['a', 'b'])
    t = db()
    d = dictable(a = [1,2,3])*dict(b = [1,2,3])
    t.reset.drop()
    t = t.insert_many(d)
    assert len(t) == 9
    t = t.insert_many(d)
    assert len(t) == 9
    assert len(t.deleted) == 9
    t.insert_one(Dict(a = 4, b = 4))
    assert len(t) == 10
    t.insert_one(Dict(a = 4, b = 4))
    assert len(t) == 10
    t.insert_one(Dict(a = 4, b = 4, c = 16))
    assert t.find_one(a = 4, b = 4)[0].c == 16
    t['c'] = lambda a, b: a + b
    assert t.find_one(a = 3, b = 3)[0].c == 6
    assert t.find(a = 3).c == [4,5,6]
    
    assert len(t.deleted[::]) == 21
    t.deleted.inc(a = 3, b = 3)[::]
    t.inc(a = 3, b = 3)[::]
    del t.sort('a', 'b')[0]
    assert len(t.deleted[::]) == 21 + 1
    assert len(t.inc(a = 1, b = 1)) == 0

    del t['c']
    assert 'c' not in t.find_one(a = 3, b = 3)[0]
    
    del t[dict(a = 3, b = 3)]
    with pytest.raises(ValueError):
        t[dict(a = 3, b = 3)]

    t.set(c = lambda a, b: a+b+5)
    assert t.find_one(a = 1, b = 2)[0].c == 8
    t.reset.drop()


def test_pk_cursor_dedup():
    db = partial(mongo_table, db = 'test', table = 'test', pk = ['a', 'b'])
    r = mongo_table('test', 'test', ['a', 'b'], mode = 'r')
    assert type(r) == mongo_reader
    t = db()
    assert type(t) == mongo_pk_cursor
    d = dictable(a = [1,2,3])*dict(b = [1,2,3])
    t.reset.drop()
    t = t.insert_many(d)
    d[_pk] = [['a', 'b']]
    t.reset.insert_many(d) ## creating duplicates
    t.reset.insert_many(d) ## creating duplicates
    assert len(t) == 27
    assert len(r.find_one(a = 1, b = 1)) == 1
    assert len(t) == 25
    t = t.dedup()
    assert len(t) == 9
    with pytest.raises(KeyError):
        r.find_one(a = 1)


def test_pk_cursor_insert_missing_keys():
    t = mongo_table('test', 'test', pk = ['a', 'b'])
    t.reset.drop()
    with pytest.raises(ValueError):
        t.insert_one(dict(a = 1))


def test_pk_cursor_update_into_insert():
    t = mongo_table('test', 'test', pk = ['a', 'b'])
    t.reset.drop()
    first = t.update_one(dict(a = 1, b = 1))
    second =  t.update_one(dict(a = 1, b = 1))
    assert second['_id'] == first['_id']
    
    
def test_pk_cursor_update_many():
    t = mongo_table('test', 'test', pk = ['a', 'b'])
    t.reset.drop()
    d = dictable(a = [1,2,3], b = [4,5,6])
    res = t.update_many(d)
    assert len(res) == 3 and len(t) == 3 and res[['a', 'b']] == d
    t.reset.drop()
    res = t.update_many(d, False)
    assert res == dictable(data =  [None] * 3)
    
def test_pk_cursor_setitem():
    t = mongo_table('test', 'test', pk = ['a', 'b'])
    t.reset.drop()
    t[dict(a = 1, b = 2)] = 3
    assert len(t) == 1 and t[0]['data'] == 3
    t[dict(a = 1, b = 2)] = dict(c = 3)
    assert len(t) == 1 and t[0]['c'] == 3
    t[dict(a = 2, b = 2)] = dict(c = 4)
    assert len(t) == 2 and t.find_one(a = 2, b = 2)[0]['c']==4


def test_pk_cursor__add__():
    t = mongo_table('test', 'test', pk = ['a', 'b'])
    t.reset.drop()
    d = dictable(a = [1,2,3], b = [4,5,6])
    t = t + d    
    assert len(t) == 3 and t[::][['a', 'b']] == d
    t = t + dict(a = 5, b = 6)
    assert len(t) == 4
    with pytest.raises(ValueError):
        t + 5
        
        
def test_pk_cursor_raw():
    t = mongo_table('test', 'test', pk = ['a', 'b'])
    assert isinstance(t.reset, mongo_cursor) and not t.reset.spec
    r = t.reset.find(a = 1)
    assert isinstance(r.reset, mongo_cursor) and not r.reset.spec

    t = mongo_table('test', 'test', pk = ['a', 'b'], mode = 'r')
    assert isinstance(t.reset, mongo_reader) and not t.reset.spec
    r = t.reset.find(a = 1)
    assert isinstance(r.reset, mongo_reader) and not r.reset.spec 

def test_pk_cursor_dates_to_parquet():
    db = partial(mongo_table, table = 'test', db =  'test', pk = 'a', writer = 'c:/temp/%a/%updated.parquet')    
    db().reset.drop()
    c = periodic_cell(lambda a: pd.Series([1,2,3], drange(-2)), a = 'a', db = db)()
    assert eq(pd_read_parquet(db().read(0, passthru)['data']['path']), c.data)
    db().reset.drop()
    c = periodic_cell(lambda a: pd.Series([1,2,3], drange(-2)), a = 'a', db = db)()
    c.updated = dt(2000)
    c = c.save()
    assert db().read(0, passthru)['data']['path'] == 'c:/temp/a/20000101/data.parquet'
    db().reset.drop()




from pyg_base import * 
from pyg_mongo import * 

from pyg_mongo import get_cell, get_data, mongo_table, db_cell, cell_push 
from pyg_base import mul_, cell, dictable, dt, get_DAG, add_, get_cache    

import pytest
from functools import partial



def test_that_GRAPH_gets_populated_when_cell_go():
    db = partial(mongo_table, db = 'test', table = 'test', pk = 'key')
    db().reset.drop()
    a = db_cell(add_, a = 1, b = 2, key = 'a', db = db).go()
    assert a._address in get_cache('GRAPH')


def test_get_cell():
    db = mongo_table(db = 'test', table = 'test')
    db.drop()
    d = (dictable(a = [1,2,3,4,5,6]) * dict(b = [1,2,3,4,5]))(data = lambda a, b: a*b)
    db.insert_many([cell(row) for row in d])

    assert get_cell('test','test', a = 1, b = 1).data == 1
    assert get_data('test','test', a = 1, b = 1) == 1
    assert get_cell('test','test', a = 3, b = 4).data == 12
    assert get_data('test','test', a = 3, b = 4) == 12
    with pytest.raises(ValueError):
        get_cell('test', 'test', a = 3)    
    with pytest.raises(ValueError):
        get_cell('test', 'test', a = 30)    
    db.drop()
    
    
def test_get_cell_with_version_control():
    db = partial(mongo_table, db = 'test', table = 'test', pk = ['a', 'b'])
    db().reset.drop()
    d = (dictable(a = [1,2,3]) * dict(b = [1,2,3]))
    c = db_cell(function = mul_, db = db)
    _ = d[lambda a, b: c(a = a, b = b)(1)]
    t = dt()

    assert get_cell('test','test', a = 1, b = 1).data == 1
    assert get_data('test','test', a = 1, b = 1) == 1
    assert get_cell('test','test', a = 3, b = 2).data == 6
    assert get_data('test','test', a = 3, b = 2) == 6
    with pytest.raises(ValueError):
        load_cell('test', 'test', a = 3)    
    with pytest.raises(ValueError):
        load_cell('test', 'test', a = 30)    
    db().drop()
    with pytest.raises(ValueError):
        load_cell('test','test', a = 1, b = 1)
    assert load_cell('test','test', a = 1, b = 1, deleted = t).data == 1
    db().reset.drop()


def test_get_cell_fail_on_history():
    db = mongo_table(db = 'test', table = 'test')
    db.reset.drop()
    db.insert_one(dict(a = 1, deleted = dt(2000)))
    with pytest.raises(ValueError):
        get_cell('test', 'test', a = 1, deleted = dt(2001))
    db.insert_one(dict(a = 1, b = 1))
    db.insert_one(dict(a = 1, b = 2))
    with pytest.raises(ValueError):
        get_cell('test', 'test', a = 1, deleted = dt(2001))


def test_cell_push_pull():
    pk = 'key'
    a = cell(add_, a = 1, b = 2, key = 'a', pk = pk)(mode = -1)
    b = cell(add_, a = a, b = 2, key = 'b', pk = pk)(mode = -1)    
    c = cell(add_, a = a, b = b, key = 'c', pk = pk)(mode = -1)
    d = cell(add_, a = c, b = b, key = 'd', pk = pk)(mode = -1)
    e = cell(add_, a = d, b = b, key = 'e', pk = pk)(mode = -1)
    GRAPH = get_cache('GRAPH')
    assert a._address in GRAPH
    assert get_data(key = 'e') == 18
    a.a = 6
    a = a.push()
    assert a.data == 8
    assert e.load().data == 38
    assert get_data(key = 'e') == 38

def test_db_cell_push_pull():
    db = partial(mongo_table, db = 'test', table = 'test', pk = 'key')
    db().reset.drop()
    a = db_cell(add_, a = 1, b = 2, key = 'a', db = db)(mode = -1)
    b = db_cell(add_, a = a, b = 2, key = 'b', db = db)(mode = -1)    
    c = db_cell(add_, a = a, b = b, key = 'c', db = db)(mode = -1)
    d = db_cell(add_, a = c, b = b, key = 'd', db = db)(mode = -1)
    e = db_cell(add_, a = d, b = b, key = 'e', db = db)(mode = -1)
    assert get_data('test', 'test', key = 'e') == 18
    a.a = 6
    a = a.push()
    assert a.data == 8
    assert get_data('test', 'test', key = 'e') == 38
    GRAPH = get_cache('GRAPH')
    assert GRAPH[e._address].data == 38
    b.b = 4
    b = b.push()
    assert GRAPH[e._address].data == 44
    assert e.data == 18
    assert e.load().data == 44
    db().reset.drop()

def test_db_cell_queued_push_pull():
    db = partial(mongo_table, db = 'test', table = 'test', pk = 'key')
    db().reset.drop()
    a = cell(add_, a = 1, b = 2, key = 'a', db = db, pk = 'key')(mode = -1)
    b = db_cell(add_, a = a, b = 2, key = 'b', db = db, pk = 'key')(mode = -1)  
    c = db_cell(add_, a = a, b = b, key = 'c', db = db, pk = 'key')(mode = -1)
    d = db_cell(add_, a = c, b = b, key = 'd', db = db, pk = 'key')(mode = -1)
    e = db_cell(add_, a = d, b = b, key = 'e', db = db, pk = 'key')(mode = -1)
    assert get_data('test', 'test', key = 'e') == 18
    get_cache('UPDATED')
    
    a.a = 6
    a = a.push()
    assert get_data('test', 'test', key = 'e') == 38
    b.b = 4
    b = b.push()
    assert get_data('test', 'test', key = 'e') == 44
    db().reset.drop()
    a = db_cell(add_, a = 1, b = 2, key = 'a', db = db)(mode = -1)
    b = db_cell(add_, a = a, b = 2, key = 'b', db = db)(mode = -1)  
    c = db_cell(add_, a = a, b = b, key = 'c', db = db)(mode = -1)
    d = db_cell(add_, a = c, b = b, key = 'd', db = db)(mode = -1)
    e = db_cell(add_, a = d, b = b, key = 'e', db = db)(mode = -1)
    a.a = 6
    a = a.go()
    cell_push()
    assert get_data('test', 'test', key = 'e') == 38
    b.b = 4
    b = b.go()
    UPDATED = get_cache('UPDATED')
    assert len(UPDATED) == 1 and b._address in UPDATED
    cell_push()
    assert get_data('test', 'test', key = 'e') == 44
    assert e.load().data == 44    
    db().reset.drop()

    